#!/usr/bin/python
# -* coding: iso-8859-1 -*-
"""
**********************************************************************
File Name:             EXPGSM485adc.py
Dev:                   Daniele De Santis
Email:                 desantis@areasx.com
Web:                   http://www.areasx.com

Versione:              0.0
Data:                  06/06/16
Note Hardware:         TELIT GL865 su GSM485
Note Software: 
**********************************************************************
"""

import sys
import MDM
import GPIO
import MOD
import SER

GPIO_RED        = 2
GPIO_GREEN      = 1
GPIO_IN1        = 4
GPIO_IN2        = 5

DEBUG_ON        = 1

#Inizializzazione GPIO
def InitGPIO():
  GPIO.setIOdir(GPIO_RED,0,1) #Output LED Rosso
  GPIO.setIOdir(GPIO_GREEN,0,1) #Output LED Verde
  GPIO.setIOdir(GPIO_IN1,0,0) #Input1
  GPIO.setIOdir(GPIO_IN2,0,0) #Input2

#Inizializzazione Seriale
def InitSER(Bps='115200'):
  SER.set_speed(Bps)

#Print dei debug
def PrintDEBUG(Txt, Mode=1):
  print Txt
  if(Mode): 
    InitSER()
    SER.send(Txt+"\r\n")

class ModBusRTU:
  def __init__(self):
    self.Debug=0
    self.Baudrate = "19200"
    self.RX_Cmd   = ""
    
  def InitSER(self):
    SER.set_speed(self.Baudrate)
    return 1
  
  def CRC(self, cmd):
    poly = 0xA001
    CRC  = 0xFFFF
    for character in cmd:
      CRC = (CRC ^ (ord(character)))
      for i in range(8):
        CRC, carrybit = (CRC>>1),(CRC&1)
        if carrybit == 1: CRC = (CRC ^ poly)
    return CRC
  
  def Printhex(self, cmd, mode=0):
    tmp=""
    for i in cmd:
      if(mode): tmp=tmp+"%02X"     % ord(i)
      else:     tmp=tmp+"0x%02X "  % ord(i)
    return tmp

  def SendCmd(self, Address, Function, Register, Registervalue, Registerqt=1, Bytecount=2):
    TX_Cmd  = chr(Address) + chr(Function) + chr((Register>>8) & 0xFF) + chr((Register & 0xFF))
    if(Registerqt>1): 
      TX_Cmd = TX_Cmd + chr((Registerqt>>8) & 0xFF) + chr((Registerqt & 0xFF))
      if(Bytecount):    TX_Cmd =TX_Cmd + chr(Bytecount)
    for i in range (1,Bytecount+1):
      TX_Cmd =TX_Cmd + chr((Registervalue>>((Bytecount-i) * 0x08) & 0xFF))
    crc= self.CRC(TX_Cmd)
    TX_Cmd = TX_Cmd + chr((crc & 0xFF)) + chr(((crc>>8) & 0xFF)) #Appendo il CRC
    self.RX_Cmd=""
    Rx_Len = 6 * int(Registerqt)
    if(Function==0x05): Rx_Len=8
    if(Function==0x04): Rx_Len=7
    SER.send(TX_Cmd)
    if(self.Debug): print "TX -> %s" % self.Printhex(TX_Cmd)
    T0 = MOD.secCounter()
    RX_Size_tmp=0
    while(1):
      self.RX_Cmd = self.RX_Cmd + SER.receive(1)
      RX_Size = len(self.RX_Cmd)
      if(RX_Size!=RX_Size_tmp): 
        T0 = MOD.secCounter()
        RX_Size_tmp = RX_Size
      if (MOD.secCounter()>(T0+1)): break #Timeout
        
    if (RX_Size>=4):
      if(self.Debug): print "RX <- %s" % self.Printhex(self.RX_Cmd)
      rx_CRC=self.RX_Cmd[RX_Size-2:RX_Size]
      my_CRC=self.CRC(self.RX_Cmd[0:-2])
      if((ord(rx_CRC[0])==(my_CRC & 0xFF)) and (ord(rx_CRC[1])==((my_CRC>>8) & 0xFF))):
        return RX_Size
      if(self.Debug): print "ERRORE - CRC"
      return -1
    return 0

##########################################################################################
#TEST acquisizione USCITE GSM485 & EXP485
InitGPIO()
EXP         = ModBusRTU()
EXP.Debug   = DEBUG_ON
EXP_Address = 0x01
EXP.Baudrate= "19200"


if(EXP.InitSER()):
  adc = {}
  if(EXP.SendCmd(EXP_Address, 0x4, 0x0001, 0x0004, 0x00, 0x02)):
    output_value = ord(EXP.RX_Cmd[3])
    adc[0] =  (ord(EXP.RX_Cmd[4]) + (ord(EXP.RX_Cmd[3])<<8))
    adc[1] =  (ord(EXP.RX_Cmd[6])  + (ord(EXP.RX_Cmd[5])<<8))
    adc[2] =  (ord(EXP.RX_Cmd[8])  + (ord(EXP.RX_Cmd[7])<<8))
    adc[3] =  (ord(EXP.RX_Cmd[10]) + (ord(EXP.RX_Cmd[9])<<8))
    

  for i in range(0, 4):
    print "ADC%d [%d]" % (i, adc[i])
